@@ -3,10 +3,13 @@ package org.jsoup.parser;
 import org.jsoup.UncheckedIOException;
 import org.jsoup.helper.Validate;
 
+import javax.annotation.Nullable;
 import java.io.IOException;
 import java.io.Reader;
 import java.io.StringReader;
+import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Collections;
 import java.util.Locale;
 
 /**
@@ -29,6 +32,9 @@ public final class CharacterReader {
     private static final int stringCacheSize = 512;
     private String[] stringCache = new String[stringCacheSize]; // holds reused strings in this doc, to lessen garbage
 
+    @Nullable private ArrayList<Integer> newlinePositions = null; // optionally track the pos() position of newlines - scans during bufferUp()
+    private int lineNumberOffset = 1; // line numbers start at 1; += newlinePosition[indexof(pos)]
+
     public CharacterReader(Reader input, int sz) {
         Validate.notNull(input);
         Validate.isTrue(input.markSupported());
@@ -98,16 +104,127 @@ public final class CharacterReader {
         } catch (IOException e) {
             throw new UncheckedIOException(e);
         }
+        scanBufferForNewlines(); // if enabled, we index newline positions for line number tracking
+        lastIcSeq = null; // cache for last containsIgnoreCase(seq)
     }
 
     /**
-     * Gets the current cursor position in the content.
+     * Gets the position currently read to in the content. Starts at 0.
      * @return current position
      */
     public int pos() {
         return readerPos + bufPos;
     }
 
+    /**
+     Enables or disables line number tracking. By default, will be <b>off</b>.Tracking line numbers improves the
+     legibility of parser error messages, for example. Tracking should be enabled before any content is read to be of
+     use.
+
+     @param track set tracking on|off
+     @since 1.14.3
+     */
+    public void trackNewlines(boolean track) {
+        if (track && newlinePositions == null) {
+            newlinePositions = new ArrayList<>(maxBufferLen / 80); // rough guess of likely count
+            scanBufferForNewlines(); // first pass when enabled; subsequently called during bufferUp
+        }
+        else if (!track)
+            newlinePositions = null;
+    }
+
+    /**
+     Check if the tracking of newlines is enabled.
+     @return the current newline tracking state
+     @since 1.14.3
+     */
+    public boolean isTrackNewlines() {
+        return newlinePositions != null;
+    }
+
+    /**
+     Get the current line number (that the reader has consumed to). Starts at line #1.
+     @return the current line number, or 1 if line tracking is not enabled.
+     @since 1.14.3
+     @see #trackNewlines(boolean)
+     */
+    public int lineNumber() {
+        return lineNumber(pos());
+    }
+
+    int lineNumber(int pos) {
+        // note that this impl needs to be called before the next buffer up or line numberoffset will be wrong. if that
+        // causes issues, can remove the reset of newlinepositions during buffer, at the cost of a larger tracking array
+        if (!isTrackNewlines())
+            return 1;
+
+        int i = lineNumIndex(pos);
+        if (i == -1)
+            return lineNumberOffset; // first line
+        return i + lineNumberOffset + 1;
+    }
+
+    /**
+     Get the current column number (that the reader has consumed to). Starts at column #1.
+     @return the current column number
+     @since 1.14.3
+     @see #trackNewlines(boolean)
+     */
+    public int columnNumber() {
+        return columnNumber(pos());
+    }
+
+    int columnNumber(int pos) {
+        if (!isTrackNewlines())
+            return pos + 1;
+
+        int i = lineNumIndex(pos);
+        if (i == -1)
+          return pos + 1;
+        return pos - newlinePositions.get(i) + 1;
+    }
+
+    /**
+     Get a formatted string representing the current line and cursor positions. E.g. <code>5:10</code> indicating line
+     number 5 and column number 10.
+     @return line:col position
+     @since 1.14.3
+     @see #trackNewlines(boolean)
+     */
+    String cursorPos() {
+        return lineNumber() + ":" + columnNumber();
+    }
+
+    private int lineNumIndex(int pos) {
+        if (!isTrackNewlines()) return 0;
+        int i = Collections.binarySearch(newlinePositions, pos);
+        if (i < -1) i = Math.abs(i) - 2;
+        return i;
+    }
+
+    /**
+     Scans the buffer for newline position, and tracks their location in newlinePositions.
+     */
+    private void scanBufferForNewlines() {
+        if (!isTrackNewlines())
+            return;
+
+        if (newlinePositions.size() > 0) {
+            // work out the line number that we have read up to (as we have likely scanned past this point)
+            int index = lineNumIndex(readerPos);
+            if (index == -1) index = 0; // first line
+            int linePos = newlinePositions.get(index);
+            lineNumberOffset += index; // the num lines we've read up to
+            newlinePositions.clear();
+            newlinePositions.add(linePos); // roll the last read pos to first, for cursor num after buffer
+        }
+
+        for (int i = bufPos; i < bufLength; i++) {
+            if (charBuf[i] == '\n')
+                newlinePositions.add(1 + readerPos + i);
+        }
+    }
+
     /**
      * Tests if all the content has been read.
      * @return true if nothing left to read.
@@ -551,11 +668,31 @@ public final class CharacterReader {
         }
     }
 
+    // we maintain a cache of the previously scanned sequence, and return that if applicable on repeated scans.
+    // that improves the situation where there is a sequence of <p<p<p<p<p<p<p...</title> and we're bashing on the <p
+    // looking for the </title>. Resets in bufferUp()
+    @Nullable private String lastIcSeq; // scan cache
+    private int lastIcIndex; // nearest found indexOf
+
+    /** Used to check presence of </title>, </style> when we're in RCData and see a <xxx. Only finds consistent case. */
     boolean containsIgnoreCase(String seq) {
-        // used to check presence of </title>, </style>. only finds consistent case.
+        if (seq.equals(lastIcSeq)) {
+            if (lastIcIndex == -1) return false;
+            if (lastIcIndex >= bufPos) return true;
+        }
+        lastIcSeq = seq;
+
         String loScan = seq.toLowerCase(Locale.ENGLISH);
+        int lo = nextIndexOf(loScan);
+        if (lo > -1) {
+            lastIcIndex = bufPos + lo; return true;
+        }
+
         String hiScan = seq.toUpperCase(Locale.ENGLISH);
-        return (nextIndexOf(loScan) > -1) || (nextIndexOf(hiScan) > -1);
+        int hi = nextIndexOf(hiScan);
+        boolean found = hi > -1;
+        lastIcIndex = found ? bufPos + hi : -1; // we don't care about finding the nearest, just that buf contains
+        return found;
     }
 
     @Override
