@@ -2,6 +2,7 @@ package org.jsoup.nodes;
 
 import org.jsoup.Jsoup;
 import org.jsoup.TextUtil;
+import org.jsoup.internal.StringUtil;
 import org.jsoup.parser.ParseSettings;
 import org.jsoup.parser.Parser;
 import org.jsoup.parser.Tag;
@@ -11,7 +12,10 @@ import org.jsoup.select.NodeFilter;
 import org.jsoup.select.NodeVisitor;
 import org.jsoup.select.QueryParser;
 import org.junit.jupiter.api.Test;
+import org.junit.jupiter.params.ParameterizedTest;
+import org.junit.jupiter.params.provider.MethodSource;
 
+import java.io.IOException;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.HashSet;
@@ -20,6 +24,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import java.util.concurrent.atomic.AtomicInteger;
+import java.util.stream.Stream;
 
 import static org.junit.jupiter.api.Assertions.*;
 
@@ -430,6 +435,38 @@ public class ElementTest {
         assertEquals("<html>\n<head></head>\n<body>\n<div>\n<p>Hello there</p>\n</div>\n</body>\n</html>", doc.html());
     }
 
+    @Test void testIndentLevel() {
+        // deep to test default and extended max
+        StringBuilder divs = new StringBuilder();
+        for (int i = 0; i < 40; i++) {
+            divs.append("<div>");
+        }
+        divs.append("Foo");
+        Document doc = Jsoup.parse(divs.toString());
+        Document.OutputSettings settings = doc.outputSettings();
+
+        int defaultMax = 30;
+        assertEquals(defaultMax, settings.maxPaddingWidth());
+        String html = doc.html();
+        assertTrue(html.contains("                              <div>\n" +
+            "                              Foo\n" +
+            "                              </div>"));
+
+        settings.maxPaddingWidth(32);
+        assertEquals(32, settings.maxPaddingWidth());
+        html = doc.html();
+        assertTrue(html.contains("                                <div>\n" +
+            "                                Foo\n" +
+            "                                </div>"));
+
+        settings.maxPaddingWidth(-1);
+        assertEquals(-1, settings.maxPaddingWidth());
+        html = doc.html();
+        assertTrue(html.contains("                                         <div>\n" +
+            "                                          Foo\n" +
+            "                                         </div>"));
+    }
+
     @Test
     public void testNotPretty() {
         Document doc = Jsoup.parse("<div>   \n<p>Hello\n there\n</p></div>");
@@ -492,7 +529,7 @@ public class ElementTest {
     public void testNoIndentOnScriptAndStyle() {
         // don't newline+indent closing </script> and </style> tags
         Document doc = Jsoup.parse("<script>one\ntwo</script>\n<style>three\nfour</style>");
-        assertEquals("<script>one\ntwo</script> \n<style>three\nfour</style>", doc.head().html());
+        assertEquals("<script>one\ntwo</script>\n<style>three\nfour</style>", doc.head().html());
     }
 
     @Test
@@ -500,7 +537,7 @@ public class ElementTest {
         Document doc = Jsoup.parse("<title>Hello there</title> <div><p>Hello</p><p>there</p></div> <div>Another</div>");
         assertEquals("<title>Hello there</title>", doc.select("title").first().outerHtml());
         assertEquals("<div>\n <p>Hello</p>\n <p>there</p>\n</div>", doc.select("div").first().outerHtml());
-        assertEquals("<div>\n <p>Hello</p>\n <p>there</p>\n</div> \n<div>\n Another\n</div>", doc.select("body").first().html());
+        assertEquals("<div>\n <p>Hello</p>\n <p>there</p>\n</div>\n<div>\n Another\n</div>", doc.select("body").first().html());
     }
 
     @Test
@@ -589,6 +626,7 @@ public class ElementTest {
         Document doc = Jsoup.parse("<div id=1><p>Hello</p></div>");
         Element div = doc.getElementById("1");
         div.appendText(" there & now >");
+        assertEquals ("Hello there & now >", div.text());
         assertEquals("<p>Hello</p> there &amp; now &gt;", TextUtil.stripNewlines(div.html()));
     }
 
@@ -833,7 +871,10 @@ public class ElementTest {
         Element p = doc.select("p").get(1);
         Element clone = p.clone();
 
-        assertNull(clone.parent()); // should be orphaned
+        assertNotNull(clone.parentNode); // should be a cloned document just containing this clone
+        assertEquals(1, clone.parentNode.childNodeSize());
+        assertSame(clone.ownerDocument(), clone.parentNode);
+
         assertEquals(0, clone.siblingIndex);
         assertEquals(1, p.siblingIndex);
         assertNotNull(p.parent());
@@ -1747,6 +1788,7 @@ public class ElementTest {
     public void testTraverse() {
         Document doc = Jsoup.parse("<div><p>One<p>Two<p>Three");
         Element div = doc.selectFirst("div");
+        assertNotNull(div);
         final AtomicInteger counter = new AtomicInteger(0);
 
         Element div2 = div.traverse(new NodeVisitor() {
@@ -1766,11 +1808,24 @@ public class ElementTest {
         assertEquals(div2, div);
     }
 
-    @Test
-    public void voidTestFilterCallReturnsElement() {
-        // doesn't actually test the filter so much as the return type for Element. See node.nodeFilter for an acutal test
+    @Test void testTraverseLambda() {
         Document doc = Jsoup.parse("<div><p>One<p>Two<p>Three");
         Element div = doc.selectFirst("div");
+        assertNotNull(div);
+        final AtomicInteger counter = new AtomicInteger(0);
+
+        Element div2 = div.traverse((node, depth) -> counter.incrementAndGet());
+
+        assertEquals(7, counter.get());
+        assertEquals(div2, div);
+    }
+
+    @Test
+    public void testFilterCallReturnsElement() {
+        // doesn't actually test the filter so much as the return type for Element. See node.nodeFilter for an actual test
+        Document doc = Jsoup.parse("<div><p>One<p>Two<p>Three");
+        Element div = doc.selectFirst("div");
+        assertNotNull(div);
         Element div2 = div.filter(new NodeFilter() {
             @Override
             public FilterResult head(Node node, int depth) {
@@ -1786,6 +1841,28 @@ public class ElementTest {
         assertSame(div, div2);
     }
 
+    @Test void testFilterAsLambda() {
+        Document doc = Jsoup.parse("<div><p>One<p id=2>Two<p>Three");
+        doc.filter((node, depth) -> node.attr("id").equals("2")
+            ? NodeFilter.FilterResult.REMOVE
+            : NodeFilter.FilterResult.CONTINUE);
+
+        assertEquals("<div><p>One</p><p>Three</p></div>", TextUtil.stripNewlines(doc.body().html()));
+    }
+
+    @Test void testForEach() {
+        Document doc = Jsoup.parse("<div><p>Hello</p></div><div>There</div><div id=1>Gone<p></div>");
+        doc.forEach(el -> {
+            if (el.id().equals("1"))
+                el.remove();
+            else if (el.text().equals("There")) {
+                el.text("There Now");
+                el.append("<p>Another</p>");
+            }
+        });
+        assertEquals("<div><p>Hello</p></div><div>There Now<p>Another</p></div>", TextUtil.stripNewlines(doc.body().html()));
+    }
+
     @Test
     public void doesntDeleteZWJWhenNormalizingText() {
         String text = "\uD83D\uDC69\u200D\uD83D\uDCBB\uD83E\uDD26\uD83C\uDFFB\u200D\u2642\uFE0F";
@@ -2066,4 +2143,195 @@ public class ElementTest {
         p.removeAttr("foo");
         assertEquals(0, p.attributesSize());
     }
-}
+
+    @Test void clonedElementsHaveOwnerDocsAndIndependentSettings() {
+        // https://github.com/jhy/jsoup/issues/763
+        Document doc = Jsoup.parse("<div>Text</div><div>Two</div>");
+        doc.outputSettings().prettyPrint(false);
+        Element div = doc.selectFirst("div");
+        assertNotNull(div);
+        Node text = div.childNode(0);
+        assertNotNull(text);
+
+        Element divClone = div.clone();
+        Document docClone = divClone.ownerDocument();
+        assertNotNull(docClone);
+        assertFalse(docClone.outputSettings().prettyPrint());
+        assertNotSame(doc, docClone);
+        assertSame(docClone, divClone.childNode(0).ownerDocument());
+        // the cloned text has same owner doc as the cloned div
+
+        doc.outputSettings().prettyPrint(true);
+        assertTrue(doc.outputSettings().prettyPrint());
+        assertFalse(docClone.outputSettings().prettyPrint());
+        assertEquals(1, docClone.children().size()); // check did not get the second div as the owner's children
+        assertEquals(divClone, docClone.child(0)); // note not the head or the body -- not normalized
+    }
+
+    private static Stream<Document.OutputSettings> testOutputSettings() {
+        return Stream.of(
+            new Document.OutputSettings().prettyPrint(true).indentAmount(4),
+            new Document.OutputSettings().prettyPrint(true).indentAmount(1),
+            new Document.OutputSettings().prettyPrint(true).indentAmount(4).outline(true),
+            new Document.OutputSettings().prettyPrint(false)
+        );
+    }
+
+    @ParameterizedTest
+    @MethodSource("testOutputSettings")
+    void prettySerializationRoundTrips(Document.OutputSettings settings) {
+        // https://github.com/jhy/jsoup/issues/1688
+        // tests that repeated html() and parse() does not accumulate errant spaces / newlines
+        Document doc = Jsoup.parse("<div>\nFoo\n<p>\nBar\nqux</p></div>\n<script>\n alert('Hello!');\n</script>");
+        doc.outputSettings(settings);
+        String html = doc.html();
+        Document doc2 = Jsoup.parse(html);
+        doc2.outputSettings(settings);
+        String html2 = doc2.html();
+
+        assertEquals(html, html2);
+    }
+
+    @Test void prettyPrintScriptsDoesNotGrowOnRepeat() {
+        Document doc = Jsoup.parse("<div>\nFoo\n<p>\nBar\nqux</p></div>\n<script>\n alert('Hello!');\n</script>");
+        Document.OutputSettings settings = doc.outputSettings();
+        settings
+            .prettyPrint(true)
+            .outline(true)
+            .indentAmount(4)
+            ;
+
+        String html = doc.html();
+        Document doc2 = Jsoup.parse(html);
+        doc2.outputSettings(settings);
+        String html2 = doc2.html();
+        assertEquals(html, html2);
+    }
+
+    @Test void elementBrText() {
+        // testcase for https://github.com/jhy/jsoup/issues/1437
+        String html = "<p>Hello<br>World</p>";
+        Document doc = Jsoup.parse(html);
+        Element p = doc.select("p").first();
+        assertNotNull(p);
+        assertEquals(html, p.outerHtml());
+        assertEquals("Hello World", p.text());
+        assertEquals("Hello\nWorld", p.wholeText());
+    }
+
+    @Test void preformatFlowsToChildTextNodes() {
+        // https://github.com/jhy/jsoup/issues/1776
+        String html = "<div><pre>One\n<span>\nTwo</span>\n <span>  \nThree</span>\n <span>Four <span>Five</span>\n  Six\n</pre>";
+        Document doc = Jsoup.parse(html);
+        doc.outputSettings().indentAmount(2).prettyPrint(true);
+
+        Element div = doc.selectFirst("div");
+        assertNotNull(div);
+        String actual = div.outerHtml();
+        String expect = "<div>\n" +
+            "  <pre>One\n" +
+            "<span>\n" +
+            "Two</span>\n" +
+            " <span>  \n" +
+            "Three</span>\n" +
+            " <span>Four <span>Five</span>\n" +
+            "  Six\n" +
+            "</span></pre>\n" +
+            "</div>";
+        assertEquals(expect, actual);
+
+        String expectText = "One\n" +
+            "\n" +
+            "Two\n" +
+            "   \n" +
+            "Three\n" +
+            " Four Five\n" +
+            "  Six\n";
+        assertEquals(expectText, div.wholeText());
+
+        String expectOwn = "One\n" +
+            "\n" +
+            " \n" +
+            " ";
+        assertEquals(expectOwn, div.child(0).wholeOwnText());
+    }
+
+    @Test void testExpectFirst() {
+        Document doc = Jsoup.parse("<p>One</p><p>Two <span>Three</span> <span>Four</span>");
+
+        Element span = doc.expectFirst("span");
+        assertEquals("Three", span.text());
+
+        assertNull(doc.selectFirst("div"));
+        boolean threw = false;
+        try {
+            Element div = doc.expectFirst("div");
+        } catch (IllegalArgumentException e) {
+            threw = true;
+        }
+        assertTrue(threw);
+    }
+
+    @Test void spanRunsMaintainSpace() {
+        // https://github.com/jhy/jsoup/issues/1787
+        Document doc = Jsoup.parse("<p><span>One</span>\n<span>Two</span>\n<span>Three</span></p>");
+        String text = "One Two Three";
+        Element body = doc.body();
+        assertEquals(text, body.text());
+
+        Element p = doc.expectFirst("p");
+        String html = p.html();
+        p.html(html);
+        assertEquals(text, body.text());
+
+        assertEquals("<p><span>One</span> <span>Two</span> <span>Three</span></p>", body.html());
+    }
+
+    @Test void doctypeIsPrettyPrinted() {
+        // resolves underlying issue raised in https://github.com/jhy/jsoup/pull/1664
+        Document doc1 = Jsoup.parse("<!--\nlicense\n-->\n \n<!doctype html>\n<html>");
+        Document doc2 = Jsoup.parse("\n  <!doctype html><html>");
+        Document doc3 = Jsoup.parse("<!doctype html>\n<html>");
+        Document doc4 = Jsoup.parse("\n<!doctype html>\n<html>");
+        Document doc5 = Jsoup.parse("\n<!--\n comment \n -->  <!doctype html>\n<html>");
+        Document doc6 = Jsoup.parse("<!--\n comment \n -->  <!doctype html>\n<html>");
+
+        assertEquals("<!--\nlicense\n-->\n<!doctype html>\n<html>\n <head></head>\n <body></body>\n</html>", doc1.html());
+        doc1.outputSettings().prettyPrint(false);
+        assertEquals("<!--\nlicense\n--><!doctype html>\n<html><head></head><body></body></html>", doc1.html());
+        // note that the whitespace between the comment and the doctype is not retained, in Initial state
+
+        assertEquals("<!doctype html>\n<html>\n <head></head>\n <body></body>\n</html>", doc2.html());
+        assertEquals("<!doctype html>\n<html>\n <head></head>\n <body></body>\n</html>", doc3.html());
+        assertEquals("<!doctype html>\n<html>\n <head></head>\n <body></body>\n</html>", doc4.html());
+        assertEquals("<!--\n comment \n -->\n<!doctype html>\n<html>\n <head></head>\n <body></body>\n</html>", doc5.html());
+        assertEquals("<!--\n comment \n -->\n<!doctype html>\n<html>\n <head></head>\n <body></body>\n</html>", doc6.html());
+    }
+
+    @Test void textnodeInBlockIndent() {
+        String html ="<div>\n{{ msg }} \n </div>\n<div>\n{{ msg }} \n </div>";
+        Document doc = Jsoup.parse(html);
+        assertEquals("<div>\n {{ msg }}\n</div>\n<div>\n {{ msg }}\n</div>", doc.body().html());
+    }
+
+    @Test void stripTrailing() {
+        String html = "<p> This <span>is </span>fine. </p>";
+        Document doc = Jsoup.parse(html);
+        assertEquals("<p>This <span>is </span>fine.</p>", doc.body().html());
+    }
+
+    @Test void elementIndentAndSpaceTrims() {
+        String html = "<body><div> <p> One Two </p> <a>  Hello </a><p>\nSome text \n</p>\n </div>";
+        Document doc = Jsoup.parse(html);
+        assertEquals("<div>\n" +
+            " <p>One Two</p> <a> Hello </a>\n" +
+            " <p>Some text</p>\n" +
+            "</div>", doc.body().html());
+    }
+
+    @Test void divAInlineable() {
+        String html = "<body><div> <a>Text</a>";
+        Document doc = Jsoup.parse(html);
+        assertEquals("<div><a>Text</a>\n</div>", doc.body().html());
+    }
+}
\ No newline at end of file
